export default GML;
/**
 * Feature format for reading and writing data in the GML format
 * version 3.1.1.
 * Currently only supports GML 3.1.1 Simple Features profile.
 *
 * @class
 * @param {import("./GMLBase.js").Options} [options]
 *     Optional configuration object.
 * @api
 */
declare const GML: typeof GML3;
import GML3 from './GML3.js';
//# sourceMappingURL=GML.d.ts.map