/**
 * @module ol/tilegrid/common
 */
/**
 * Default maximum zoom for default tile grids.
 * @type {number}
 */
export const DEFAULT_MAX_ZOOM: number;
/**
 * Default tile size.
 * @type {number}
 */
export const DEFAULT_TILE_SIZE: number;
//# sourceMappingURL=common.d.ts.map