/**
 * Compute world params
 * @param {import("../../Map.js").FrameState} frameState Frame state.
 * @param {any} layer The layer
 * @return {Array<number>} The world start, end and width.
 */
export function getWorldParameters(frameState: import("../../Map.js").FrameState, layer: any): Array<number>;
//# sourceMappingURL=worldUtil.d.ts.map