declare namespace _default {
    let ADDFEATURE: string;
    let CHANGEFEATURE: string;
    let CLEAR: string;
    let REMOVEFEATURE: string;
    let FEATURESLOADSTART: string;
    let FEATURESLOADEND: string;
    let FEATURESLOADERROR: string;
}
export default _default;
export type VectorSourceEventTypes = "addfeature" | "changefeature" | "clear" | "removefeature" | "featuresloadstart" | "featuresloadend" | "featuresloaderror";
//# sourceMappingURL=VectorEventType.d.ts.map