declare namespace _default {
    let PRERENDER: string;
    let POSTRENDER: string;
    let PRECOMPOSE: string;
    let POSTCOMPOSE: string;
    let RENDERCOMPLETE: string;
}
export default _default;
export type MapRenderEventTypes = "postrender" | "precompose" | "postcompose" | "rendercomplete";
export type LayerRenderEventTypes = "postrender" | "prerender";
//# sourceMappingURL=EventType.d.ts.map