export default CanvasPolygonBuilder;
declare class CanvasPolygonBuilder extends CanvasBuilder {
    /**
     * @param {Array<number>} flatCoordinates Flat coordinates.
     * @param {number} offset Offset.
     * @param {Array<number>} ends Ends.
     * @param {number} stride Stride.
     * @private
     * @return {number} End.
     */
    private drawFlatCoordinatess_;
    /**
     * @private
     */
    private setFillStrokeStyles_;
}
import CanvasBuilder from './Builder.js';
//# sourceMappingURL=PolygonBuilder.d.ts.map