export default IsNull;
/**
 * @classdesc
 * Represents a `<PropertyIsNull>` comparison operator.
 * @api
 */
declare class IsNull extends Comparison {
    /**
     * @param {!string} propertyName Name of the context property to compare.
     */
    constructor(propertyName: string);
}
import Comparison from './Comparison.js';
//# sourceMappingURL=IsNull.d.ts.map