/**
 * @module ol/obj
 */
/**
 * Removes all properties from an object.
 * @param {Object<string, unknown>} object The object to clear.
 */
export function clear(object: {
    [x: string]: unknown;
}): void;
/**
 * Determine if an object has any properties.
 * @param {Object} object The object to check.
 * @return {boolean} The object is empty.
 */
export function isEmpty(object: any): boolean;
//# sourceMappingURL=obj.d.ts.map