/**
 * @module ol/asserts
 */
/**
 * @param {*} assertion Assertion we expected to be truthy.
 * @param {string} errorMessage Error message.
 */
export function assert(assertion: any, errorMessage: string): void;
//# sourceMappingURL=asserts.d.ts.map