/**
 * User agent string says we are dealing with Firefox as browser.
 * @type {boolean}
 */
export const FIREFOX: boolean;
/**
 * User agent string says we are dealing with Safari as browser.
 * @type {boolean}
 */
export const SAFARI: boolean;
/**
 * https://bugs.webkit.org/show_bug.cgi?id=237906
 * @type {boolean}
 */
export const SAFARI_BUG_237906: boolean;
/**
 * User agent string says we are dealing with a WebKit engine.
 * @type {boolean}
 */
export const WEBKIT: boolean;
/**
 * User agent string says we are dealing with a Mac as platform.
 * @type {boolean}
 */
export const MAC: boolean;
/**
 * The ratio between physical pixels and device-independent pixels
 * (dips) on the device (`window.devicePixelRatio`).
 * @const
 * @type {number}
 * @api
 */
export const DEVICE_PIXEL_RATIO: number;
/**
 * The execution context is a worker with OffscreenCanvas available.
 * @const
 * @type {boolean}
 */
export const WORKER_OFFSCREEN_CANVAS: boolean;
/**
 * Image.prototype.decode() is supported.
 * @type {boolean}
 */
export const IMAGE_DECODE: boolean;
/**
 * createImageBitmap() is supported.
 * @type {boolean}
 */
export const CREATE_IMAGE_BITMAP: boolean;
/**
 * @type {boolean}
 */
export const PASSIVE_EVENT_LISTENERS: boolean;
//# sourceMappingURL=has.d.ts.map