declare namespace _default {
    let SINGLECLICK: string;
    let CLICK: string;
    let DBLCLICK: string;
    let POINTERDRAG: string;
    let POINTERMOVE: string;
    let POINTERDOWN: string;
    let POINTERUP: string;
    let POINTEROVER: string;
    let POINTEROUT: string;
    let POINTERENTER: string;
    let POINTERLEAVE: string;
    let POINTERCANCEL: string;
}
export default _default;
/**
 * *
 */
export type Types = "singleclick" | "click" | "dblclick" | "pointerdrag" | "pointermove";
//# sourceMappingURL=MapBrowserEventType.d.ts.map