/**
 * @module ol/geom/flat/flip
 */
/**
 * @param {Array<number>} flatCoordinates Flat coordinates.
 * @param {number} offset Offset.
 * @param {number} end End.
 * @param {number} stride Stride.
 * @param {Array<number>} [dest] Destination.
 * @param {number} [destOffset] Destination offset.
 * @return {Array<number>} Flat coordinates.
 */
export function flipXY(flatCoordinates: Array<number>, offset: number, end: number, stride: number, dest?: Array<number>, destOffset?: number): Array<number>;
//# sourceMappingURL=flip.d.ts.map