export default CanvasLineStringBuilder;
declare class CanvasLineStringBuilder extends CanvasBuilder {
    /**
     * @param {Array<number>} flatCoordinates Flat coordinates.
     * @param {number} offset Offset.
     * @param {number} end End.
     * @param {number} stride Stride.
     * @private
     * @return {number} end.
     */
    private drawFlatCoordinates_;
}
import CanvasBuilder from './Builder.js';
//# sourceMappingURL=LineStringBuilder.d.ts.map