/**
 * @module ol/resolution
 */
/**
 * @typedef {number|Array<number>} ResolutionLike
 */
/**
 * @param {ResolutionLike} resolution Resolution.
 * @return {number} Resolution.
 */
export function fromResolutionLike(resolution: ResolutionLike): number;
export type ResolutionLike = number | Array<number>;
//# sourceMappingURL=resolution.d.ts.map