/**
 * @module ol/source/common
 */
/**
 * Default WMS version.
 * @type {string}
 */
export const DEFAULT_WMS_VERSION: string;
/**
 * Number of decimal digits to consider in integer values when rounding.
 * @type {number}
 */
export const DECIMALS: number;
//# sourceMappingURL=common.d.ts.map